@@ -282,6 +282,14 @@ def ArgsForClusterRef(
       help='The type of cluster.',
   )
 
+  parser.add_argument(
+      '--tier',
+      hidden=True,
+      metavar='TYPE',
+      choices=['premium', 'standard'],
+      help='The tier of cluster.',
+  )
+
   image_parser = parser.add_mutually_exclusive_group()
   # TODO(b/73291743): Add external doc link to --image
   image_parser.add_argument(
@@ -819,18 +827,13 @@ If you want to enable all scopes use the 'cloud-platform' scope.
     parser.add_argument(
         '--stop-max-idle',
         type=arg_parsers.Duration(),
-        # TODO: b/368979261 - Unhide scheduled stop flags after release.
-        hidden=True,
         help="""\
           The duration after the last job completes to auto-stop the cluster,
           such as "2h" or "1d".
           See $ gcloud topic datetimes for information on duration formats.
           """,
     )
-    auto_stop_group = parser.add_mutually_exclusive_group(
-        # TODO: b/368979261 - Unhide scheduled stop flags after release.
-        hidden=True,
-    )
+    auto_stop_group = parser.add_mutually_exclusive_group()
     auto_stop_group.add_argument(
         '--stop-max-age',
         type=arg_parsers.Duration(),
@@ -1424,6 +1427,7 @@ def GetClusterConfig(
       configBucket=args.bucket,
       tempBucket=args.temp_bucket,
       clusterType=_GetCusterType(dataproc, args.cluster_type),
+      clusterTier=_GetClusterTier(dataproc, args.tier),
       gceClusterConfig=gce_cluster_config,
       masterConfig=dataproc.messages.InstanceGroupConfig(
           numInstances=args.num_masters,
@@ -1916,6 +1920,34 @@ def _GetCusterType(dataproc, cluster_type):
   )
 
 
+def _GetClusterTier(dataproc, cluster_tier):
+  """Get ClusterTier enum value.
+
+  Converts cluster_tier argument value to
+  ClusterTier API enum value.
+
+  Args:
+    dataproc: Dataproc API definition
+    cluster_tier: argument value
+
+  Returns:
+    ClusterTier API enum value
+  """
+  if cluster_tier == 'premium':
+    return dataproc.messages.ClusterConfig.ClusterTierValueValuesEnum(
+        'CLUSTER_TIER_PREMIUM'
+    )
+  if cluster_tier == 'standard':
+    return dataproc.messages.ClusterConfig.ClusterTierValueValuesEnum(
+        'CLUSTER_TIER_STANDARD'
+    )
+  if cluster_tier is None:
+    return None
+  raise exceptions.ArgumentError(
+      'Unsupported --cluster-tier flag value: ' + cluster_tier
+  )
+
+
 def _GetPrivateIpv6GoogleAccess(dataproc, private_ipv6_google_access_type):
   """Get PrivateIpv6GoogleAccess enum value.
 
